/* ERD - Copyright (C) 2005 bakunin - Andrea Marchesini 
 *                              <bakunin@autistici.org>
 *
 * This source code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Public License as published 
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This source code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * Please refer to the GNU Public License for more details.
 *
 * You should have received a copy of the GNU Public License along with
 * this source code; if not, write to:
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#else
# error Use configure; make; make install
#endif

#include "erd.h"

void
command_action (struct erd_config *conf)
{
  char **arg = command_parse (conf->input_command);
  struct erd_item input;
  pid_t pid;
  int p_in[2];
  int p_out[2];

  if (pipe (p_in))
    fatal ("Pipe error.");

  if (pipe (p_out))
    fatal ("Pipe error.");

  switch ((pid = fork ()))
    {
    case -1:
      fatal ("Forking error.");
      break;

    case 0:
      close (p_out[0]);
      close (STDOUT_FILENO);

      close (p_in[1]);
      close (STDIN_FILENO);

      dup2 (p_out[1], STDOUT_FILENO);
      dup2 (p_in[0], STDIN_FILENO);

      execve (arg[0], arg, conf->env);

      close (p_out[1]);
      close (p_in[0]);
      exit (1);

      break;

    default:
      close (p_out[1]);
      close (p_in[0]);

      input.fd = p_out[0];
      input.pid = pid;
      input.fd_input = p_in[1];

      input.use_ssl = conf->input_ssl;

      if (conf->input_ssl)
	{
	  input.ssl = SSL_new (ssl_get_ctx ());

	  if (!input.ssl)
	    fatal ("SSL create error.");

	  SSL_set_fd (input.ssl, input.fd);

	  if (SSL_accept (input.ssl) < 0)
	    {
	      log_write (ERD_ERROR, "SSL client error.");
	      wait (NULL);
	      return;
	    }
	}

      erd_redirect (conf, &input);

      command_free_arg (arg);

      log_write (ERD_WARN, "Closing connection...");

      if (input.use_ssl)
	{
	  SSL_shutdown (input.ssl);
	  SSL_free (input.ssl);
	}

      close (input.fd);
      close(input.fd_input);

      break;
    }
}


struct command_list
{
  char *arg;
  struct command_list *next;
} command_list;

char **
command_parse (char *a)
{
  int k = 0;
  char opt[1024];
  struct command_list *head, *tail, *tmp;
  int len = 0;
  char **arg;

  head = tail = NULL;

  log_write (ERD_INFO, "command_parse: %s", a);

  while (*a)
    if (*a == ' ' || *a == '\t')
      {
	a++;
	if (k)
	  {
	    opt[k] = 0;

	    if (!
		(tmp =
		 (struct command_list *)
		 malloc (sizeof (struct command_list))))
	      fatal ("Error: memory.");

	    if (!(tmp->arg = strdup (opt)))
	      fatal ("Error: memory.");

	    tmp->next = NULL;
	    len++;

	    if (!head)
	      head = tail = tmp;
	    else
	      {
		tail->next = tmp;
		tail = tmp;
	      }

	    k = 0;

	  }
      }

    else
      while (*a && *a != ' ' && *a != '\t')
	if (k < sizeof (opt))
	  opt[k++] = *a++;

  if (k)
    {
      opt[k] = 0;

      if (!
	  (tmp =
	   (struct command_list *) malloc (sizeof (struct command_list))))
	fatal ("Error: memory.");

      if (!(tmp->arg = strdup (opt)))
	fatal ("Error: memory.");

      tmp->next = NULL;
      len++;

      if (!head)
	head = tail = tmp;
      else
	{
	  tail->next = tmp;
	  tail = tmp;
	}
    }

  if (!(arg = (char **) malloc (sizeof (char *) * (len + 1))))
    fatal ("Error: memory.");
  len = 0;

  tmp = head;
  while (tmp)
    {
      arg[len] = tmp->arg;

      tmp = tmp->next;
      len++;
    }

  arg[len] = NULL;

  while (head)
    {
      tmp = head;
      head = head->next;

      free (tmp);
    }

  return arg;
}

void
command_free_arg (char **arg)
{
  int len = 0;

  while (arg[len])
    {
      free (arg[len]);
      len++;
    }
}

/* EOF */
