/* ERD - Copyright (C) 2005 bakunin - Andrea Marchesini 
 *                              <bakunin@autistici.org>
 *
 * This source code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Public License as published 
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This source code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * Please refer to the GNU Public License for more details.
 *
 * You should have received a copy of the GNU Public License along with
 * this source code; if not, write to:
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#else
# error Use configure; make; make install
#endif

#include "erd.h"

struct erd_config *
config (char *config_file)
{
  struct erd_config *conf;
  cfg_t *cfg = NULL;

  cfg_opt_t opts[] = {
    CFG_STR ("LogFile", NULL, CFGF_NONE),
    CFG_BOOL ("Background", cfg_false, CFGF_NONE),
    CFG_INT ("Debug", 2, CFGF_NONE),

    CFG_STR ("Input_Type", NULL, CFGF_NONE),

    CFG_STR ("Input_Command", NULL, CFGF_NONE),

    CFG_STR ("Input_UnixPath", NULL, CFGF_NONE),
    CFG_STR ("Input_ServerName", NULL, CFGF_NONE),
    CFG_INT ("Input_Port", ERD_PORT, CFGF_NONE),
    CFG_INT ("Input_Listen", ERD_LISTEN, CFGF_NONE),

    CFG_BOOL ("Input_Ssl", cfg_false, CFGF_NONE),
    CFG_STR ("Input_Certificate", NULL, CFGF_NONE),
    CFG_STR ("Input_PrivateKey", NULL, CFGF_NONE),

    CFG_STR ("Output_Type", NULL, CFGF_NONE),

    CFG_STR ("Output_Command", NULL, CFGF_NONE),

    CFG_STR ("Output_UnixPath", NULL, CFGF_NONE),
    CFG_STR ("Output_Server", NULL, CFGF_NONE),
    CFG_INT ("Output_Port", ERD_PORT, CFGF_NONE),

    CFG_BOOL ("Output_Ssl", cfg_false, CFGF_NONE),
    CFG_END ()
  };

  if (!(conf = (struct erd_config *) malloc (sizeof (struct erd_config))))
    fatal ("Error: memory.");
  memset (conf, 0, sizeof (struct erd_config));

  cfg = cfg_init (opts, CFGF_NOCASE);
  switch (cfg_parse (cfg, config_file))
    {
    case CFG_FILE_ERROR:
      fatal ("Error: open file: %s", config_file);
      break;
    case CFG_PARSE_ERROR:
      fatal ("Error: syntax error: %s", config_file);
      break;
    }

  conf->logfile = cfg_getstr (cfg, "LogFile");
  if (!conf->logfile || !*conf->logfile)
    conf->logfile = strdup (ERD_LOG_FILE);
  else
    conf->logfile = strdup (conf->logfile);
  if (!conf->logfile)
    fatal ("Error: memory.");

  conf->background = cfg_getbool (cfg, "Background");

  conf->debug = cfg_getint (cfg, "Debug");

  conf->input_type = config_type (cfg_getstr (cfg, "Input_Type"));

  conf->input_ssl = cfg_getbool (cfg, "Input_Ssl");

  conf->input_certificate = cfg_getstr (cfg, "Input_Certificate");
  if (!conf->input_certificate || !*conf->input_certificate)
    conf->input_certificate = NULL;
  else
    conf->input_certificate = strdup (conf->input_certificate);

  conf->input_privatekey = cfg_getstr (cfg, "input_PrivateKey");
  if (!conf->input_privatekey || !*conf->input_privatekey)
    conf->input_privatekey = NULL;
  else
    conf->input_privatekey = strdup (conf->input_privatekey);

  if (conf->input_ssl
      && (!conf->input_certificate || !conf->input_privatekey))
    fatal
      ("Set Input_Certificate and Input_PrivateKey flags for Input_Ssl support.");

  conf->input_unixpath = cfg_getstr (cfg, "Input_UnixPath");
  if (!conf->input_unixpath || !*conf->input_unixpath)
    conf->input_unixpath = strdup (ERD_UNIX_SOCKET);
  else
    conf->input_unixpath = strdup (conf->input_unixpath);
  if (!conf->input_unixpath)
    fatal ("Error: memory.");

  conf->input_servername = cfg_getstr (cfg, "Input_ServerName");
  if (!conf->input_servername || !*conf->input_servername)
    conf->input_servername = NULL;
  else if (!(conf->input_servername = strdup (conf->input_servername)))
    fatal ("Error: memory.");

  conf->input_port = cfg_getint (cfg, "Input_Port");
  conf->input_listen = cfg_getint (cfg, "Input_Listen");

  conf->input_command = cfg_getstr (cfg, "Input_Command");
  if (!conf->input_command || !*conf->input_command)
    conf->input_command = NULL;
  else
    conf->input_command = strdup (conf->input_command);

  conf->output_ssl = cfg_getbool (cfg, "Output_Ssl");

  conf->output_type = config_type (cfg_getstr (cfg, "Output_Type"));

  conf->output_unixpath = cfg_getstr (cfg, "Output_UnixPath");
  if (!conf->output_unixpath || !*conf->output_unixpath)
    conf->output_unixpath = strdup (ERD_UNIX_SOCKET);
  else
    conf->output_unixpath = strdup (conf->output_unixpath);
  if (!conf->output_unixpath)
    fatal ("Error: memory.");

  conf->output_server = cfg_getstr (cfg, "Output_Server");
  if (!conf->output_server || !*conf->output_server)
    conf->output_server = strdup ("localhost");
  else
    conf->output_server = strdup (conf->output_server);
  if (!conf->output_server)
    fatal ("Error: memory.");

  conf->output_port = cfg_getint (cfg, "Output_Port");

  conf->output_command = cfg_getstr (cfg, "Output_Command");
  if (!conf->output_command || !*conf->output_command)
    conf->output_command = NULL;
  else
    conf->output_command = strdup (conf->output_command);

  cfg_free (cfg);

  if ((conf->input_type == ERD_COMMAND && !conf->input_command))
    fatal ("Set the input command.");

  if ((conf->output_type == ERD_COMMAND && !conf->output_command))
    fatal ("Set the output command.");

  return conf;
}

int
config_type (char *c)
{
  if (!c)
    fatal ("Error in parameter type.");

  if (!strcasecmp (c, "u") || !strcasecmp (c, "unix"))
    return ERD_UNIXSOCKET;

  else if (!strcasecmp (c, "t") || !strcasecmp (c, "tcp"))
    return ERD_TCPSOCKET;

  else if (!strcasecmp (c, "c") || !strcasecmp (c, "command"))
    return ERD_COMMAND;

  else if (!strcasecmp (c, "f") || !strcasecmp (c, "filedescriptor"))
    return ERD_FD;

  else
    fatal ("Error in parameter: %s as type.", c);

  return 0;
}

/* EOF */
