/* ERD - Copyright (C) 2005 bakunin - Andrea Marchesini 
 *                             <bakunin@autistici.org>
 *
 * This source code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Public License as published 
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This source code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * Please refer to the GNU Public License for more details.
 *
 * You should have received a copy of the GNU Public License along with
 * this source code; if not, write to:
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __ERD_H__
#define __ERD_H__

#include <unistd.h>
#include <fcntl.h>
#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <dirent.h>
#include <pthread.h>
#include <errno.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <termios.h>

#include <openssl/rsa.h>
#include <openssl/crypto.h>
#include <openssl/ssl.h>
#include <openssl/err.h>
#include <openssl/x509.h>
#include <openssl/pem.h>

#include "../confuse/confuse.h"

#define ERD_LISTEN		5
#define ERD_PORT		12721
#define ERD_CONFIG_FILE		"/etc/erd.cfg"
#define ERD_UNIX_SOCKET		"/tmp/erd.sock"
#define ERD_LOG_FILE		"/var/log/erd.log"
#define AUTHOR			"Andrea Marchesini <bakunin@autistici.org>"

#define ERD_NO_LOG		0
#define ERD_ERROR		1
#define ERD_WARN		2
#define ERD_INFO		3

#define ERD_UNIXSOCKET		0
#define ERD_TCPSOCKET		1
#define ERD_COMMAND		2
#define ERD_FD			3

struct erd_config {
  char **env;

  int background;
  char *logfile;
  int debug;

  int input_type;

  char *input_unixpath;
  char *input_servername;
  int input_port;
  int input_listen;

  int input_ssl;
  char *input_certificate;
  char *input_privatekey;

  char *input_command;

  int output_type;

  char *output_unixpath;
  char *output_server;
  int output_port;

  int output_ssl;

  char *output_command;

} erd_config;

struct erd_item
{
  SSL *ssl;

  int fd;
  int pid;
  int fd_input;

  int use_ssl; 

} erd_item;

struct socket_client
{
  pthread_t th;

  struct erd_item *sock;

  struct erd_config *data;

  int flag;
} socket_client;

/* main.c */
void fatal(char *, ...);
void signal_on (void);
void signal_off (void);

/* config.c */
struct erd_config *config(char *);
int config_type(char *c);

/* socker.c */
int socket_open(struct erd_config *);
void socket_new_client(struct erd_config *, int);

/* redirect.c */
void erd_redirect(struct erd_config *, struct erd_item *);
void erd_close (struct erd_item *);

/* log.c */
void log_init(struct erd_config *);
void log_quit(void);
void log_write(int, char *,...);
int log_is_opened(void);

/* io.c */
struct erd_config *io_config (char *, char *);

/* command.c */
void command_action(struct erd_config *);
char **command_parse (char *);
void command_free_arg (char **);

/* ssl.h */
void ssl_init(void);
void ssl_certificate(struct erd_config *);
SSL_CTX *ssl_get_ctx(void);

#endif
