/* ERD - Copyright (C) 2005 bakunin - Andrea Marchesini 
 *                              <bakunin@autistici.org>
 *
 * This source code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Public License as published 
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This source code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * Please refer to the GNU Public License for more details.
 *
 * You should have received a copy of the GNU Public License along with
 * this source code; if not, write to:
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#else
# error Use configure; make; make install
#endif

#include "erd.h"

static void *socket_thread (void *);

int
socket_open (struct erd_config *conf)
{
  int fd = 0, yes = 1;
  struct sockaddr_in sock;
  struct hostent *hp;
  struct sockaddr_un saddr;

  if (conf->input_type == ERD_TCPSOCKET)
    {
      if ((fd = socket (AF_INET, SOCK_STREAM, 0)) < 0)
	fatal ("Sock error.");

      if (setsockopt (fd, SOL_SOCKET, SO_REUSEADDR, &yes, sizeof (int)))
	fatal ("Sock error.");

      memset ((void *) &sock, 0, sizeof (sock));
      sock.sin_family = AF_INET;
      sock.sin_port = htons (conf->input_port);

      if (!conf->input_servername)
	sock.sin_addr.s_addr = htonl (INADDR_ANY);

      else
	{
	  if (!(hp = gethostbyname (conf->input_servername)))
	    fatal ("Unknow %s", conf->input_servername);

	  sock.sin_family = hp->h_addrtype;
	  memcpy ((caddr_t) & sock.sin_addr, hp->h_addr, hp->h_length);
	}

      if (bind (fd, (struct sockaddr *) &sock, sizeof (sock)) < 0)
	fatal ("Binding error.");

    }

  else if (conf->input_type == ERD_UNIXSOCKET)
    {
      char path[sizeof (saddr.sun_path)];
      struct stat st;

      if ((fd = socket (AF_UNIX, SOCK_STREAM, 0)) < 0)
	fatal ("Sock error.");

      saddr.sun_family = AF_UNIX;

      strncpy (path, conf->input_unixpath, sizeof (path));

      if (!lstat (path, &st))
	{
	  unlink (path);

	  if (!lstat (path, &st))
	    fatal ("Error opening %s socket", path);
	}

      strncpy (saddr.sun_path, path, sizeof (saddr.sun_path));

      if (bind (fd, (struct sockaddr *) &saddr, sizeof (saddr)) < 0)
	fatal ("Binding error.");
    }

  if (listen (fd, conf->input_listen) < 0)
    fatal ("Listen error.");

  return fd;
}

void
socket_new_client (struct erd_config *conf, int fd)
{
  struct socket_client *cl;

  if (!(cl = (struct socket_client *) malloc (sizeof (struct socket_client))))
    fatal ("Error: memory.");

  memset (cl, 0, sizeof (struct socket_client));

  if (!(cl->sock = (struct erd_item *) malloc (sizeof (struct erd_item))))
    fatal ("Error: memory.");

  cl->sock->fd = fd;
  cl->sock->pid = 0;
  cl->data = conf;

  if (pthread_create (&cl->th, NULL, socket_thread, cl))
    fatal ("Threading error.");

  if (pthread_detach (cl->th))
    fatal ("Threading error.");
}

static void *
socket_thread (void *dummy)
{
  struct socket_client *c = (struct socket_client *) dummy;

  signal_off ();

  /* Switch to ssl */
  if (c->data->input_ssl)
    {
      c->sock->ssl = SSL_new (ssl_get_ctx ());
      c->sock->use_ssl = 1;

      if (!c->sock->ssl)
	fatal ("SSL create error.");

      SSL_set_fd (c->sock->ssl, c->sock->fd);

      if (SSL_accept (c->sock->ssl) < 0)
	{
	  log_write (ERD_ERROR, "SSL client error.");
	  erd_close (c->sock);

	  c->flag = 1;
	  signal_on ();
	  pthread_exit (NULL);
	  return 0;
	}

      log_write (ERD_INFO, "New connect with SSL: %s.",
		 SSL_get_cipher (c->sock->ssl));
    }

  else
    c->sock->use_ssl = 0;

  erd_redirect (c->data, c->sock);

  log_write (ERD_WARN, "Closing connection...");
  erd_close (c->sock);

  c->flag = 1;
  signal_on ();
  pthread_exit (NULL);
  return 0;
}

/* EOF */
